@@ -1,14 +1,15 @@
+use crate::fl;
 use crate::{
     config::{Config, Lang, Profile, SslMode, TableStyle},
-    mysql::connect,
+    mysql::Session,
     output::Format,
     utils::read_file,
 };
-use crate::{fl, output::QueryOutput};
 use anyhow::{anyhow, Context, Result};
 use std::io::Write;
 use structopt::StructOpt;
 
+mod http;
 pub mod shell;
 
 #[cfg_attr(feature = "zh-CN", doc = "数据库连接工具")]
@@ -95,6 +96,30 @@ pub enum DCliCommand {
         #[structopt(short, long)]
         profile: String,
     },
+
+    #[cfg_attr(feature = "zh-CN", doc = "运行一个 HTTP　服务器以展示，下载数据")]
+    #[cfg_attr(feature = "en-US", doc = "run a HTTP server to display or download data")]
+    Serve {
+        #[cfg_attr(feature = "zh-CN", doc = "连接配置名称")]
+        #[cfg_attr(feature = "en-US", doc = "profile name")]
+        #[structopt(short, long)]
+        profile: String,
+
+        #[cfg_attr(feature = "zh-CN", doc = "port 1 ~ 65535")]
+        #[cfg_attr(feature = "en-US", doc = "port 1 ~ 65535")]
+        #[structopt(default_value = "3030", short = "P", long)]
+        port: u16,
+
+        #[cfg_attr(
+            feature = "zh-CN",
+            doc = "命令 使用 @<文件路径> 读取 SQL 文件内容作为输入"
+        )]
+        #[cfg_attr(
+            feature = "en-US",
+            doc = "sql, use @<file_path> to read SQL file as input"
+        )]
+        command: Vec<String>,
+    },
 }
 
 #[derive(Debug, StructOpt)]
@@ -222,7 +247,7 @@ impl DCliCommand {
             }
             DCliCommand::Exec { profile, command } => {
                 let profile = config.try_get_profile(profile)?;
-                let pool = connect(&profile).await?;
+                let session = Session::connect_with(&profile).await?;
                 let to_execute = if command.len() == 1 && command.first().unwrap().starts_with('@')
                 {
                     read_file(&command.first().unwrap()[1..])?
@@ -231,11 +256,11 @@ impl DCliCommand {
                 };
                 for sql in to_execute.split(';') {
                     if !sql.is_empty() {
-                        let output: QueryOutput = sqlx::query(sql).fetch_all(&pool).await?.into();
+                        let output = session.query(sql).await?;
                         println!("{}", output.to_print_table(&config));
                     }
                 }
-                pool.close().await;
+                session.close().await;
                 Ok(())
             }
             DCliCommand::Export {
@@ -244,7 +269,7 @@ impl DCliCommand {
                 format,
             } => {
                 let profile = config.try_get_profile(profile)?;
-                let pool = connect(&profile).await?;
+                let session = Session::connect_with(&profile).await?;
                 let to_execute = if command.len() == 1 && command.first().unwrap().starts_with('@')
                 {
                     read_file(&command.first().unwrap()[1..])?
@@ -260,10 +285,7 @@ impl DCliCommand {
                 } else if to_execute.len() > 1 {
                     return Err(anyhow!(fl!("too-many-input")));
                 } else {
-                    let output: QueryOutput = sqlx::query(to_execute.first().unwrap())
-                        .fetch_all(&pool)
-                        .await?
-                        .into();
+                    let output = session.query(to_execute.first().unwrap()).await?;
                     match format {
                         Format::Csv => {
                             let out = output.to_csv()?;
@@ -373,6 +395,33 @@ impl DCliCommand {
                 Ok(())
             }
             DCliCommand::Shell { profile } => shell::Shell::run(config, profile).await,
+            DCliCommand::Serve {
+                profile,
+                port,
+                command,
+            } => {
+                let profile = config.try_get_profile(profile)?;
+                let session = Session::connect_with(&profile).await?;
+                let to_execute = if command.len() == 1 && command.first().unwrap().starts_with('@')
+                {
+                    read_file(&command.first().unwrap()[1..])?
+                } else {
+                    command.join(" ")
+                };
+                let to_execute = to_execute
+                    .split(';')
+                    .filter(|sql| !sql.is_empty())
+                    .collect::<Vec<&str>>();
+                if to_execute.is_empty() {
+                    return Err(anyhow!(fl!("empty-input")));
+                } else if to_execute.len() > 1 {
+                    return Err(anyhow!(fl!("too-many-input")));
+                } else {
+                    let output = session.query(to_execute.first().unwrap()).await?;
+                    http::serve(*port, output).await;
+                    Ok(())
+                }
+            }
         }
     }
 }
